@@ -631,7 +631,7 @@ ffi.metatype( knot_pkt_t, {
 		-- Question
 		qname = function(pkt)
 			assert(ffi.istype(knot_pkt_t, pkt))
-			-- inlined knot_pkt_qname(), basically
+			-- inlined knot_pkt_qname(), basically but not lower-cased
 			if pkt == nil or pkt.qname_size == 0 then return nil end
 			return ffi.string(pkt.wire + 12, pkt.qname_size)
 		end,
@@ -1135,7 +1135,7 @@ kres = {
 		if ret ~= 1 then return nil end
 		return ffi.string(addr_buf, C.kr_family_len(family))
 	end,
-	context = function () return ffi.C.the_worker.engine.resolver end,
+	context = function () return ffi.C.the_resolver end,
 
 	knot_pkt_rr = knot_pkt_rr,
 }
