@@ -10,9 +10,16 @@
 #include "daemon/network.h"
 #include "daemon/udp_queue.h"
 #include "daemon/worker.h"
+
+#ifdef ENABLE_DOH2
+#include "daemon/http.h"
+#endif
+
 #include "lib/defines.h"
 #include "lib/dnssec.h"
 #include "lib/log.h"
+#include "lib/resolve.h"
+#include "lib/rules/api.h"
 
 #include <arpa/inet.h>
 #include <getopt.h>
@@ -56,6 +63,21 @@ KR_EXPORT void kr_jemalloc_unused(void)
 KR_EXPORT const char *malloc_conf = "narenas:1";
 #endif
 
+/**@internal Maximum number of incomplete TCP connections in queue.
+* Default is from empirical testing - in our case, more isn't necessarily better.
+* See https://gitlab.nic.cz/knot/knot-resolver/-/merge_requests/968
+* */
+#ifndef TCP_BACKLOG_DEFAULT
+#define TCP_BACKLOG_DEFAULT 128
+#endif
+
+/** I don't know why linker is dropping these functions otherwise. TODO: revisit. */
+KR_EXPORT void kr_misc_unused(void)
+{
+	kr_rule_zonefile(NULL);
+	kr_rule_local_address(NULL, NULL, false, 0, 0);
+}
+
 struct args the_args_value;  /** Static allocation for the_args singleton. */
 
 static void signal_handler(uv_signal_t *handle, int signum)
@@ -153,7 +175,7 @@ static void help(int argc, char *argv[])
 }
 
 /** \return exit code for main()  */
-static int run_worker(uv_loop_t *loop, struct engine *engine, bool leader, struct args *args)
+static int run_worker(uv_loop_t *loop, bool leader, struct args *args)
 {
 	/* Only some kinds of stdin work with uv_pipe_t.
 	 * Otherwise we would abort() from libuv e.g. with </dev/null */
@@ -375,11 +397,11 @@ static int bind_sockets(addr_array_t *addrs, bool tls, flagged_fd_array_t *fds)
 	return has_error ? EXIT_FAILURE : kr_ok();
 }
 
-static int start_listening(struct network *net, flagged_fd_array_t *fds) {
+static int start_listening(flagged_fd_array_t *fds) {
 	int some_bad_ret = 0;
 	for (size_t i = 0; i < fds->len; ++i) {
 		flagged_fd_t *ffd = &fds->at[i];
-		int ret = network_listen_fd(net, ffd->fd, ffd->flags);
+		int ret = network_listen_fd(ffd->fd, ffd->flags);
 		if (ret != 0) {
 			some_bad_ret = ret;
 			/* TODO: try logging address@port.  It's not too important,
@@ -425,9 +447,9 @@ int main(int argc, char **argv)
 {
 	kr_log_group_reset();
 	if (setvbuf(stdout, NULL, _IONBF, 0) || setvbuf(stderr, NULL, _IONBF, 0)) {
-		kr_log_error(SYSTEM, "failed to set output buffering (ignored): %s\n",
+		kr_log_error(SYSTEM, "failed to to set output buffering (ignored): %s\n",
 				strerror(errno));
-		(void)fflush(stderr);
+		fflush(stderr);
 	}
 	if (strcmp("linux", OPERATING_SYSTEM) != 0)
 		kr_log_warning(SYSTEM, "Knot Resolver is tested on Linux, other platforms might exhibit bugs.\n"
@@ -490,7 +512,7 @@ int main(int argc, char **argv)
 	if (ret) {
 		kr_log_error(SYSTEM, "failed to get or set file-descriptor limit: %s\n",
 				strerror(errno));
-	} else if (rlim.rlim_cur < (rlim_t)512 * 1024) {
+	} else if (rlim.rlim_cur < 512*1024) {
 		kr_log_warning(SYSTEM, "warning: hard limit for number of file-descriptors is only %ld but recommended value is 524288\n",
 				(long)rlim.rlim_cur);
 	}
@@ -503,17 +525,23 @@ int main(int argc, char **argv)
 
 	kr_crypto_init();
 
+	network_init(uv_default_loop(), TCP_BACKLOG_DEFAULT);
+
 	/* Create a server engine. */
-	knot_mm_t pool;
-	mm_ctx_mempool(&pool, MM_DEFAULT_BLKSIZE);
-	static struct engine engine;
-	ret = engine_init(&engine, &pool);
+	ret = engine_init();
 	if (ret != 0) {
 		kr_log_error(SYSTEM, "failed to initialize engine: %s\n", kr_strerror(ret));
 		return EXIT_FAILURE;
 	}
-	/* Initialize the worker */
-	ret = worker_init(&engine, the_args->forks);
+
+	/* Create resolver context. */
+	ret = kr_resolver_init(&the_engine->modules, &the_engine->pool);
+	if (ret != 0) {
+		kr_log_error(SYSTEM, "failed to initialize resolver: %s\n", kr_strerror(ret));
+		return EXIT_FAILURE;
+	}
+	/* Initialize the worker. */
+	ret = worker_init();
 	if (ret != 0) {
 		kr_log_error(SYSTEM, "failed to initialize worker: %s\n", kr_strerror(ret));
 		return EXIT_FAILURE;
@@ -555,8 +583,14 @@ int main(int argc, char **argv)
 				uv_strerror(ret));
 	}
 
+	io_protolayers_init();
+	tls_protolayers_init();
+#ifdef ENABLE_DOH2
+	http_protolayers_init();
+#endif
+
 	/* Start listening, in the sense of network_listen_fd(). */
-	if (start_listening(&engine.net, &the_args->fds) != 0) {
+	if (start_listening(&the_args->fds) != 0) {
 		ret = EXIT_FAILURE;
 		goto cleanup;
 	}
@@ -570,42 +604,58 @@ int main(int argc, char **argv)
 	}
 
 	/* Start the scripting engine */
-	if (engine_load_sandbox(&engine) != 0) {
+	if (engine_load_sandbox() != 0) {
 		ret = EXIT_FAILURE;
 		goto cleanup;
 	}
 
 	for (i = 0; i < the_args->config.len; ++i) {
 		const char *config = the_args->config.at[i];
-		if (engine_loadconf(&engine, config) != 0) {
+		if (engine_loadconf(config) != 0) {
 			ret = EXIT_FAILURE;
 			goto cleanup;
 		}
-		lua_settop(engine.L, 0);
+		lua_settop(the_engine->L, 0);
+	}
+
+	ret = kr_rules_init_ensure();
+	if (ret) {
+		kr_log_error(RULES, "failed to initialize policy rule engine: %s\n",
+				kr_strerror(ret));
+		ret = EXIT_FAILURE;
+		goto cleanup;
 	}
 
 	drop_capabilities();
 
-	if (engine_start(&engine) != 0) {
+	if (engine_start() != 0) {
 		ret = EXIT_FAILURE;
 		goto cleanup;
 	}
 
-	if (network_engage_endpoints(&engine.net)) {
+	if (network_engage_endpoints()) {
 		ret = EXIT_FAILURE;
 		goto cleanup;
 	}
 
+	/* Starting everything succeeded, so commit rule DB changes. */
+	kr_rules_commit(true);
+
 	/* Run the event loop */
-	ret = run_worker(loop, &engine, fork_id == 0, the_args);
+	ret = run_worker(loop, fork_id == 0, the_args);
 
 cleanup:/* Cleanup. */
-	engine_deinit(&engine);
+	network_unregister();
+
+	kr_resolver_deinit();
 	worker_deinit();
+	engine_deinit();
+	network_deinit();
+	kr_rules_commit(false);
+	kr_rules_deinit();
 	if (loop != NULL) {
 		uv_loop_close(loop);
 	}
-	mp_delete(pool.ctx);
 cleanup_args:
 	args_deinit(the_args);
 	kr_crypto_cleanup();
