@@ -2,13 +2,11 @@
  *  SPDX-License-Identifier: GPL-3.0-or-later
  */
 
-#include "daemon/session.h"
 #include "daemon/network.h"
+#include "lib/generic/trie.h"
 
 #include "daemon/proxyv2.h"
 
-#include "lib/generic/trie.h"
-
 const char PROXY2_SIGNATURE[12] = {
 	0x0D, 0x0A, 0x0D, 0x0A, 0x00, 0x0D, 0x0A, 0x51, 0x55, 0x49, 0x54, 0x0A
 };
@@ -131,25 +129,25 @@ static inline void next_tlv(struct proxy2_tlv **tlv)
 }
 
 
-bool proxy_allowed(const struct network *net, const struct sockaddr *saddr)
+bool proxy_allowed(const struct sockaddr *saddr)
 {
 	union kr_in_addr addr;
 	trie_t *trie;
 	size_t addr_size;
 	switch (saddr->sa_family) {
 	case AF_INET:
-		if (net->proxy_all4)
+		if (the_network->proxy_all4)
 			return true;
 
-		trie = net->proxy_addrs4;
+		trie = the_network->proxy_addrs4;
 		addr_size = sizeof(addr.ip4);
 		addr.ip4 = ((struct sockaddr_in *)saddr)->sin_addr;
 		break;
 	case AF_INET6:
-		if (net->proxy_all6)
+		if (the_network->proxy_all6)
 			return true;
 
-		trie = net->proxy_addrs6;
+		trie = the_network->proxy_addrs6;
 		addr_size = sizeof(addr.ip6);
 		addr.ip6 = ((struct sockaddr_in6 *)saddr)->sin6_addr;
 		break;
@@ -169,7 +167,7 @@ bool proxy_allowed(const struct network *net, const struct sockaddr *saddr)
 	return kr_bitcmp((char *)&addr, (char *)&found->addr, found->netmask) == 0;
 }
 
-ssize_t proxy_process_header(struct proxy_result *out, struct session *s,
+ssize_t proxy_process_header(struct proxy_result *out,
 		const void *buf, const ssize_t nread)
 {
 	if (!buf)
@@ -279,7 +277,6 @@ ssize_t proxy_process_header(struct proxy_result *out, struct session *s,
 				&addr->ipv6_addr.dst_addr,
 				sizeof(out->dst_addr.ip6.sin6_addr.s6_addr));
 		break;
-	default:; /* Keep zero from initializer. */
 	}
 
 	/* Process additional information */
@@ -288,10 +285,10 @@ ssize_t proxy_process_header(struct proxy_result *out, struct session *s,
 		case TLV_TYPE_SSL:
 			out->has_tls = true;
 			break;
-		default:; /* Ignore others - add more if needed */
+		/* TODO: add more TLV types if needed */
 		}
 	}
 
 fill_wirebuf:
-	return session_wirebuf_trim(s, hdr_len);
+	return hdr_len;
 }
