@@ -30,10 +30,10 @@
  * So it takes 2 RTT.
  * As we use session tickets, there are additional messages, add one RTT mode.
  */
- #define TLS_MAX_HANDSHAKE_TIME (KR_CONN_RTT_MAX * (uint64_t)3)
+#define TLS_MAX_HANDSHAKE_TIME (KR_CONN_RTT_MAX * 3)
 
 /** Transport session (opaque). */
-struct session;
+struct session2;
 
 struct tls_ctx;
 struct tls_client_ctx;
@@ -74,8 +74,8 @@ typedef trie_t tls_client_params_t;
 
 /** Get a pointer-to-pointer to TLS auth params.
  * If it didn't exist, it returns NULL (if !do_insert) or pointer to NULL. */
-tls_client_param_t ** tls_client_param_getptr(tls_client_params_t **params,
-				const struct sockaddr *addr, bool do_insert);
+tls_client_param_t **tls_client_param_getptr(tls_client_params_t **params,
+		const struct sockaddr *addr, bool do_insert);
 
 /** Get a pointer to TLS auth params or NULL. */
 static inline tls_client_param_t *
@@ -94,117 +94,17 @@ int tls_client_param_remove(tls_client_params_t *params, const struct sockaddr *
 /** Free TLS authentication parameters. */
 void tls_client_params_free(tls_client_params_t *params);
 
-
-struct worker_ctx;
-struct qr_task;
-struct network;
-struct engine;
-
-typedef enum tls_client_hs_state {
-	TLS_HS_NOT_STARTED = 0,
-	TLS_HS_IN_PROGRESS,
-	TLS_HS_DONE,
-	TLS_HS_CLOSING,
-	TLS_HS_LAST
-} tls_hs_state_t;
-
-typedef int (*tls_handshake_cb) (struct session *session, int status);
-
-
-struct tls_common_ctx {
-	bool client_side;
-	gnutls_session_t tls_session;
-	tls_hs_state_t handshake_state;
-	struct session *session;
-	/* for reading from the network */
-	const uint8_t *buf;
-	ssize_t nread;
-	ssize_t consumed;
-	uint8_t recv_buf[16384];
-	tls_handshake_cb handshake_cb;
-	struct worker_ctx *worker;
-};
-
-struct tls_ctx {
-	/*
-	 * Since pointer to tls_ctx needs to be casted
-	 * to  tls_ctx_common in some functions,
-	 * this field must be always at first position
-	 */
-	struct tls_common_ctx c;
-	struct tls_credentials *credentials;
-};
-
-struct tls_client_ctx {
-	/*
-	 * Since pointer to tls_client_ctx needs to be casted
-	 * to  tls_ctx_common in some functions,
-	 * this field must be always at first position
-	 */
-	struct tls_common_ctx c;
-	tls_client_param_t *params; /**< It's reference-counted. */
-};
-
-/*! Create an empty TLS context in query context */
-struct tls_ctx* tls_new(struct worker_ctx *worker);
-
-/*! Close a TLS context (call gnutls_bye()) */
-void tls_close(struct tls_common_ctx *ctx);
-
-/*! Close a TLS client context (call gnutls_bye()), storing its session data
- * for potential resumption. */
-void tls_client_close(struct tls_client_ctx *ctx);
-
-/*! Release a TLS context */
-void tls_free(struct tls_ctx* tls);
-
-/*! Push new data to TLS context for sending */
-int tls_write(uv_handle_t* handle, knot_pkt_t * pkt);
-
-/*! Unwrap incoming data from a TLS stream and pass them to TCP session.
- * @return the number of newly-completed requests (>=0) or an error code
- */
-ssize_t tls_process_input_data(struct session *s, const uint8_t *buf, ssize_t nread);
-
 /*! Set TLS certificate and key from files. */
-int tls_certificate_set(struct network *net, const char *tls_cert, const char *tls_key);
-
-/*! Borrow TLS credentials for context. */
-struct tls_credentials *tls_credentials_reserve(struct tls_credentials *tls_credentials);
+int tls_certificate_set(const char *tls_cert, const char *tls_key);
 
 /*! Release TLS credentials for context (decrements refcount or frees). */
 int tls_credentials_release(struct tls_credentials *tls_credentials);
 
-/*! Free TLS credentials, must not be called if it holds positive refcount. */
-void tls_credentials_free(struct tls_credentials *tls_credentials);
-
-/*! Log DNS-over-TLS OOB key-pin form of current credentials:
- * https://tools.ietf.org/html/rfc7858#appendix-A */
-void tls_credentials_log_pins(struct tls_credentials *tls_credentials);
-
 /*! Generate new ephemeral TLS credentials. */
-struct tls_credentials * tls_get_ephemeral_credentials(struct engine *engine);
-
-/*! Get TLS handshake state. */
-tls_hs_state_t tls_get_hs_state(const struct tls_common_ctx *ctx);
-
-/*! Set TLS handshake state. */
-int tls_set_hs_state(struct tls_common_ctx *ctx, tls_hs_state_t state);
-
-
-/*! Allocate new client TLS context */
-struct tls_client_ctx *tls_client_ctx_new(tls_client_param_t *entry,
-					    struct worker_ctx *worker);
-
-/*! Free client TLS context */
-void tls_client_ctx_free(struct tls_client_ctx *ctx);
-
-int tls_client_connect_start(struct tls_client_ctx *client_ctx,
-			     struct session *session,
-			     tls_handshake_cb handshake_cb);
-
-int tls_client_ctx_set_session(struct tls_client_ctx *ctx, struct session *session);
+struct tls_credentials * tls_get_ephemeral_credentials(void);
 
+/*! Initializes the protocol layers managed by tls. */
+void tls_protolayers_init(void);
 
 /* Session tickets, server side.  Implementation in ./tls_session_ticket-srv.c */
 
@@ -232,3 +132,6 @@ void tls_session_ticket_enable(struct tls_session_ticket_ctx *ctx, gnutls_sessio
 /*! Free all resources of the session ticket context.  NULL is accepted as well. */
 void tls_session_ticket_ctx_destroy(struct tls_session_ticket_ctx *ctx);
 
+/*! Free TLS credentials. */
+void tls_credentials_free(struct tls_credentials *tls_credentials);
+
