@@ -8,9 +8,12 @@
 struct kr_request;
 struct qr_task;
 
+typedef void (*udp_queue_cb)(int status, void *baton);
+
 /** Initialize the global state for udp_queue. */
 int udp_queue_init_global(uv_loop_t *loop);
 
 /** Send req->answer via UDP, possibly not immediately. */
-void udp_queue_push(int fd, struct kr_request *req, struct qr_task *task);
+void udp_queue_push(int fd, const struct sockaddr *sa, char *buf, size_t buf_len,
+                    udp_queue_cb cb, void *baton);
 
