@@ -32,10 +32,15 @@
 #include <libknot/rrset.h>
 #include <libzscanner/scanner.h>
 
-#include <libdnssec/digest.h>
-#if KNOT_VERSION_HEX < 0x030200
-	#define KNOT_ZONEMD_ALGORITHM_SHA384 KNOT_ZONEMD_ALORITHM_SHA384
-	#define KNOT_ZONEMD_ALGORITHM_SHA512 KNOT_ZONEMD_ALORITHM_SHA512
+#include <libknot/version.h>
+#define ENABLE_ZONEMD (KNOT_VERSION_HEX >= 0x030100)
+#if ENABLE_ZONEMD
+	#include <libdnssec/digest.h>
+
+	#if KNOT_VERSION_HEX < 0x030200
+		#define KNOT_ZONEMD_ALGORITHM_SHA384 KNOT_ZONEMD_ALORITHM_SHA384
+		#define KNOT_ZONEMD_ALGORITHM_SHA512 KNOT_ZONEMD_ALORITHM_SHA512
+	#endif
 #endif
 
 #include "daemon/worker.h"
@@ -67,6 +72,7 @@ struct zone_import_ctx {
 	struct kr_svldr_ctx *svldr; /// DNSSEC validator; NULL iff we don't validate
 	const knot_dname_t *last_cut; /// internal to zi_rrset_import()
 
+#if ENABLE_ZONEMD
 	uint8_t *digest_buf; /// temporary buffer for digest computation (on pool)
 	#define DIGEST_BUF_SIZE (64*1024 - 1)
 	#define DIGEST_ALG_COUNT 2
@@ -75,6 +81,7 @@ struct zone_import_ctx {
 		dnssec_digest_ctx_t *ctx;
 		const uint8_t *expected; /// expected digest (inside zonemd on pool)
 	} digests[DIGEST_ALG_COUNT]; /// we use indices 0 and 1 for SHA 384 and 512
+#endif
 };
 
 typedef struct zone_import_ctx zone_import_ctx_t;
@@ -91,7 +98,7 @@ static int key_get(char buf[KEY_LEN], const knot_dname_t *name,
 	char *lf = (char *)knot_dname_lf(name, (uint8_t *)buf);
 	if (kr_fails_assert(lf && key_p))
 		return kr_error(EINVAL);
-	int len = (unsigned char)lf[0];
+	int len = lf[0];
 	lf++;  // point to start of data
 	*key_p = lf;
 	// Check that LF is right-aligned to KNOT_DNAME_MAXLEN in buf.
@@ -123,6 +130,7 @@ static knot_rrset_t * rrset_get(trie_t *rrsets, const knot_dname_t *name,
 	return *rrsig_p;
 }
 
+#if ENABLE_ZONEMD
 static int digest_rrset(trie_val_t *rr_p, void *z_import_v)
 {
 	zone_import_ctx_t *z_import = z_import_v;
@@ -274,7 +282,7 @@ do_digest:
 		// hexdump the hash for logging
 		char hash_str[digs[i].size * 2 + 1];
 		for (ssize_t j = 0; j < digs[i].size; ++j)
-			(void)sprintf(hash_str + 2*j, "%02x", digs[i].data[j]);
+			sprintf(hash_str + 2*j, "%02x", digs[i].data[j]);
 
 		if (!z_import->digests[i].expected) {
 			kr_log_error(PREFILL, "no ZONEMD found; computed hash: %s\n",
@@ -295,6 +303,8 @@ do_digest:
 	bool ok = has_match && (zonemd_is_valid || !z_import->svldr);
 	return ok ? kr_ok() : kr_error(ENOENT);
 }
+#endif
+
 
 /**
  * @internal Import given rrset to cache.
@@ -378,7 +388,7 @@ static int zi_rrset_import(trie_val_t *rr_p, void *z_import_v)
 		rank = KR_RANK_AUTH|KR_RANK_INSECURE;
 	}
 
-	int ret = kr_cache_insert_rr(&the_worker->engine->resolver.cache, rr, rrsig,
+	int ret = kr_cache_insert_rr(&the_resolver->cache, rr, rrsig,
 					rank, z_import->timestamp_rr,
 					// Optim.: only stash NSEC* params at the apex.
 					origin_bailiwick == 0);
@@ -413,7 +423,7 @@ static void zi_zone_process(uv_timer_t *timer)
 	kr_timer_start(&stopwatch);
 
 	int ret = trie_apply(z_import->rrsets, zi_rrset_import, z_import);
-	(void)kr_cache_commit(&the_worker->engine->resolver.cache); // RW transaction open
+	(void)kr_cache_commit(&the_resolver->cache); // RW transaction open
 	if (ret == 0) {
 		kr_log_info(PREFILL, "performance: validating and caching took %.3lf s\n",
 			kr_timer_elapsed(&stopwatch));
@@ -550,7 +560,7 @@ int zi_zone_import(const zi_config_t config)
 	if (kr_fails_assert(c && c->zone_file))
 		return kr_error(EINVAL);
 
-	knot_mm_t *pool = mm_ctx_mempool2((size_t)1024 * 1024);
+	knot_mm_t *pool = mm_ctx_mempool2(1024 * 1024);
 	zone_import_ctx_t *z_import = mm_calloc(pool, 1, sizeof(*z_import));
 	if (!z_import) return kr_error(ENOMEM);
 	z_import->pool = pool;
@@ -639,11 +649,10 @@ int zi_zone_import(const zi_config_t config)
    //// Initialize validator context with the DNSKEY.
 	if (c->downgrade)
 		goto zonemd;
-	struct kr_context *resolver = &the_worker->engine->resolver;
 	const knot_rrset_t * const ds = c->ds ? c->ds :
-		kr_ta_get(resolver->trust_anchors, z_import->origin);
+		kr_ta_get(the_resolver->trust_anchors, z_import->origin);
 	if (!ds) {
-		if (!kr_ta_closest(resolver, z_import->origin, KNOT_RRTYPE_DNSKEY))
+		if (!kr_ta_closest(the_resolver, z_import->origin, KNOT_RRTYPE_DNSKEY))
 			goto zonemd; // our TAs say we're insecure
 		kr_log_error(PREFILL, "no DS found for `%s`, fail\n", zone_name_str);
 		ret = kr_error(ENOENT);
@@ -696,9 +705,15 @@ int zi_zone_import(const zi_config_t config)
 zonemd: (void)0; // C can't have a variable definition following a label
 	double time_zonemd = NAN;
 	if (c->zonemd) {
-		kr_timer_start(&stopwatch);
-		ret = zonemd_verify(z_import);
-		time_zonemd = kr_timer_elapsed(&stopwatch);
+		#if ENABLE_ZONEMD
+			kr_timer_start(&stopwatch);
+			ret = zonemd_verify(z_import);
+			time_zonemd = kr_timer_elapsed(&stopwatch);
+		#else
+			kr_log_error(PREFILL,
+				"ZONEMD check requested but not supported, fail\n");
+			ret = kr_error(ENOSYS);
+		#endif
 	} else {
 		ret = kr_ok();
 	}
