@@ -4,6 +4,7 @@
 
 #pragma once
 
+#include <stdbool.h>
 #include <stdint.h>
 
 #include <libknot/db/db.h>
@@ -12,6 +13,7 @@
 struct kr_cdb_opts {
 	const char *path; /*!< Cache URI path. */
 	size_t maxsize;   /*!< Suggested cache size in bytes; pass 0 to keep unchanged/default. */
+	bool is_cache;    /*!< Some behavior changes based on use case.  TODO: details. */
 };
 
 struct kr_cdb_stats {
@@ -30,6 +32,7 @@ struct kr_cdb_stats {
 	uint64_t match_miss;
 	uint64_t read_leq;
 	uint64_t read_leq_miss;
+	uint64_t read_less;
 	double usage_percent;
 };
 
@@ -54,8 +57,11 @@ struct kr_cdb_api {
 	int (*count)(kr_cdb_pt db, struct kr_cdb_stats *stat);
 	int (*clear)(kr_cdb_pt db, struct kr_cdb_stats *stat);
 
-	/** Run after a row of operations to release transaction/lock if needed. */
-	int (*commit)(kr_cdb_pt db, struct kr_cdb_stats *stat);
+	/** Run after a row of operations to release transaction/lock if needed.
+	 * \param accept true=commit / false=abort
+	 * \return error code - accepting RW transactions can fail with LMDB.
+	 */
+	int (*commit)(kr_cdb_pt db, struct kr_cdb_stats *stat, bool accept);
 
 	/* Data access */
 
@@ -83,6 +89,12 @@ struct kr_cdb_api {
 	int (*read_leq)(kr_cdb_pt db, struct kr_cdb_stats *stat,
 			knot_db_val_t *key, knot_db_val_t *val);
 
+	/** Less-than search (lexicographic ordering).
+	 * On successful return, key->data and val->data point to DB-owned data.
+	 * return: > 0 for less, < 0 kr_error */
+	int (*read_less)(kr_cdb_pt db, struct kr_cdb_stats *stat,
+			knot_db_val_t *key, knot_db_val_t *val);
+
 	/** Return estimated space usage (0--100). */
 	double (*usage_percent)(kr_cdb_pt db);
 
@@ -94,4 +106,15 @@ struct kr_cdb_api {
 	 * and reopens it if it isn't; it errors out if the file doesn't exist anymore.
 	 * \return 0 if OK, 1 if reopened OK, < 0 kr_error */
 	int (*check_health)(kr_cdb_pt db, struct kr_cdb_stats *stat);
+
+
+	/** Start iterating; return the first *val with *key.
+	 *
+	 * This only makes sense if !is_cache.
+	 * TODO: it only works inside RO transactions for now.
+	 */
+	int (*it_first)(kr_cdb_pt db, struct kr_cdb_stats *stat,
+			const knot_db_val_t *key, knot_db_val_t *val);
+	/** Advance to the next *val with the same key. */
+	int (*it_next)(kr_cdb_pt db, struct kr_cdb_stats *stat, knot_db_val_t *val);
 };
