@@ -28,9 +28,9 @@ const knot_dname_t * kr_ta_closest(const struct kr_context *ctx, const knot_dnam
 	kr_require(ctx && name);
 	if (type == KNOT_RRTYPE_DS && name[0] != '\0') {
 		/* DS is parent-side record, so the parent name needs to be covered. */
-		name = knot_dname_next_label(name);
+		name = knot_wire_next_label(name, NULL);
 	}
-	do {
+	while (name) {
 		struct kr_context *ctx_nc = (struct kr_context *)/*const-cast*/ctx;
 		if (kr_ta_get(ctx_nc->trust_anchors, name)) {
 			return name;
@@ -38,12 +38,9 @@ const knot_dname_t * kr_ta_closest(const struct kr_context *ctx, const knot_dnam
 		if (kr_ta_get(ctx_nc->negative_anchors, name)) {
 			return NULL;
 		}
-		if (name[0] == '\0') {
-			return NULL;
-		} else {
-			name = knot_dname_next_label(name);
-		}
-	} while (true);
+		name = knot_wire_next_label(name, NULL);
+	}
+	return NULL;
 }
 
 /* @internal Create DS from DNSKEY, caller MUST free dst if successful. */
@@ -59,14 +56,13 @@ static int dnskey2ds(dnssec_binary_t *dst, const knot_dname_t *owner, const uint
 	/* Accept only keys with Zone and SEP flags that aren't revoked,
 	 * as a precaution.  RFC 5011 also utilizes these flags.
 	 * TODO: kr_dnssec_key_* names are confusing. */
-	const bool flags_ok = kr_dnssec_key_zsk(rdata) && !kr_dnssec_key_revoked(rdata);
-	if (!flags_ok) {
+	if (!kr_dnssec_key_usable(rdata)) {
 		auto_free char *owner_str = kr_dname_text(owner);
 		kr_log_error(TA, "refusing to trust %s DNSKEY because of flags %d\n",
 			owner_str, dnssec_key_get_flags(key));
 		ret = kr_error(EILSEQ);
 		goto cleanup;
-	} else if (!kr_dnssec_key_ksk(rdata)) {
+	} else if (!kr_dnssec_key_sep_flag(rdata)) {
 		auto_free char *owner_str = kr_dname_text(owner);
 		int flags = dnssec_key_get_flags(key);
 		kr_log_warning(TA, "warning: %s DNSKEY is missing the SEP bit; "
