@@ -51,7 +51,7 @@ static const knot_dname_t *minimized_qname(struct kr_query *query, uint16_t *qty
 	int cut_labels = knot_dname_labels(query->zone_cut.name, NULL);
 	int qname_labels = knot_dname_labels(qname, NULL);
 	while(qname[0] && qname_labels > cut_labels + 1) {
-		qname = knot_dname_next_label(qname);
+		qname = knot_wire_next_label(qname, NULL);
 		qname_labels -= 1;
 	}
 
@@ -825,10 +825,7 @@ static int process_answer(knot_pkt_t *pkt, struct kr_request *req)
 		}
 	} else if (!query->parent) {
 		/* Answer for initial query */
-		const bool to_wire = ((pkt_class & (PKT_NXDOMAIN|PKT_NODATA)) != 0)
-			/* We need to cover the case of positive wildcard answer
-			 * with over-limit NSEC3 iterations. */
-				|| query->flags.DNSSEC_WEXPAND;
+		const bool to_wire = ((pkt_class & (PKT_NXDOMAIN|PKT_NODATA)) != 0);
 		state = pick_authority(pkt, req, to_wire);
 		if (state != kr_ok()) {
 			return KR_STATE_FAIL;
@@ -935,6 +932,7 @@ static int begin(kr_layer_t *ctx)
 	return reset(ctx);
 }
 
+/* LATER: make calls of this function more organized; spaghetti is though here. */
 int kr_make_query(struct kr_query *query, knot_pkt_t *pkt)
 {
 	/* Minimize QNAME (if possible). */
@@ -980,6 +978,7 @@ static int prepare_query(kr_layer_t *ctx, knot_pkt_t *pkt)
 		return KR_STATE_FAIL;
 	}
 
+	// TODO: this logging (and rplan's) is confusing, especially around `uid` values
 	WITH_VERBOSE(query) {
 		KR_DNAME_GET_STR(name_str, query->sname);
 		KR_RRTYPE_GET_STR(type_str, query->stype);
@@ -1057,8 +1056,6 @@ static int resolve(kr_layer_t *ctx, knot_pkt_t *pkt)
 	if (!query) {
 		return ctx->state;
 	}
-	if (pkt->size == KR_PKT_SIZE_NOWIRE)
-		goto skip_checks; // answers from cache are sane, surely
 	query->flags.PKT_IS_SANE = false;
 
 	WITH_VERBOSE(query) {
@@ -1088,7 +1085,7 @@ static int resolve(kr_layer_t *ctx, knot_pkt_t *pkt)
 		VERBOSE_MSG("<= malformed response (parsed %d)\n", (int)pkt->parsed);
 		query->server_selection.error(query, req->upstream.transport, KR_SELECTION_MALFORMED);
 		return KR_STATE_FAIL;
-	} else if (!is_paired_to_query(pkt, query)) {
+	} else if (!query->flags.CACHED && !is_paired_to_query(pkt, query)) {
 		WITH_VERBOSE(query) {
 			const char *ns_str =
 				req->upstream.transport ? kr_straddr(&req->upstream.transport->address.ip) : "(internal)";
@@ -1110,7 +1107,7 @@ static int resolve(kr_layer_t *ctx, knot_pkt_t *pkt)
 		}
 		return KR_STATE_CONSUME;
 	}
-skip_checks:
+
 	/* If exiting above here, there's no sense to put it into packet cache.
 	 * Having "extra bytes" at the end of DNS message is considered SANE here.
 	 * The most important part is to check for spoofing: is_paired_to_query() */
