@@ -78,6 +78,8 @@ const log_group_names_t log_group_names[] = {
 	GRP_NAME_ITEM(LOG_GRP_DEVEL),
 	GRP_NAME_ITEM(LOG_GRP_RENUMBER),
 	GRP_NAME_ITEM(LOG_GRP_EDE),
+	GRP_NAME_ITEM(LOG_GRP_RULES),
+	GRP_NAME_ITEM(LOG_GRP_PROTOLAYER),
 	GRP_NAME_ITEM(LOG_GRP_REQDBG),
 	{ NULL, LOG_GRP_UNKNOWN },
 };
@@ -124,7 +126,7 @@ void kr_log_fmt(enum kr_log_group group, kr_log_level_t level, const char *file,
 		}
 
 		va_start(args, fmt);
-		(void)vfprintf(stream, fmt, args);
+		vfprintf(stream, fmt, args);
 		va_end(args);
 	}
 }
