@@ -16,6 +16,7 @@
 #include "lib/rplan.h"
 #include "lib/module.h"
 #include "lib/cache/api.h"
+#include "lib/rules/api.h"
 
 /**
  * @file resolve.h
@@ -162,6 +163,9 @@ struct kr_context
 
 	trie_t *trust_anchors;
 	trie_t *negative_anchors;
+	/** Validator's limit on the number of cryptographic steps for a single upstream packet. */
+	int32_t vld_limit_crypto;
+
 	struct kr_zonecut root_hints;
 	struct kr_cache cache;
 	unsigned cache_rtt_tout_retry_interval;
@@ -174,7 +178,18 @@ struct kr_context
 	knot_mm_t *pool;
 };
 
-/* Kept outside, because kres-gen.lua can't handle this depth
+/** Pointer to the singleton resolver context. NULL if not initialized */
+KR_EXPORT extern struct kr_context *the_resolver;
+
+/** Initializes the resolver. */
+KR_EXPORT
+int kr_resolver_init(module_array_t *modules, knot_mm_t *pool);
+
+/** Cleans up the resolver. */
+KR_EXPORT
+void kr_resolver_deinit(void);
+
+/* Kept outside struct kr_request, because kres-gen.lua can't handle this depth
  * (and lines here were too long anyway). */
 struct kr_request_qsource_flags {
 	bool tcp:1; /**< true if the request is not on UDP; only meaningful if (dst_addr). */
@@ -270,6 +285,7 @@ struct kr_request {
 	unsigned int count_no_nsaddr;
 	unsigned int count_fail_row;
 	alloc_wire_f alloc_wire_cb; /**< CB to allocate answer wire (can be NULL). */
+	kr_rule_tags_t rule_tags; /**< TagSet applying to this request. */
 	struct kr_extended_error extended_error;  /**< EDE info; don't modify directly, use kr_request_set_extended_error() */
 };
 
@@ -328,10 +344,13 @@ int kr_resolve_consume(struct kr_request *request, struct kr_transport **transpo
 /**
  * Produce either next additional query or finish.
  *
- * If the CONSUME is returned then dst, type and packet will be filled with
+ * If the CONSUME is returned then *transport and *packet will be filled with
  * appropriate values and caller is responsible to send them and receive answer.
  * If it returns any other state, then content of the variables is undefined.
  *
+ * Implemented in its own file ./resolve-produce.c
+ * FIXME: more issues in this doc-comment
+ *
  * @param  request request state (in PRODUCE state)
  * @param  dst     [out] possible address of the next nameserver
  * @param  type    [out] possible used socket type (SOCK_STREAM, SOCK_DGRAM)
@@ -348,8 +367,7 @@ int kr_resolve_produce(struct kr_request *request, struct kr_transport **transpo
  *
  * @param  request request state (in PRODUCE state)
  * @param  src     address from which the query is going to be sent
- * @param  dst     address of the name server
- * @param  type    used socket type (SOCK_STREAM, SOCK_DGRAM)
+ * @param  transport  destination server's address and other properties
  * @param  packet  [in,out] query packet to be finalised
  * @return         kr_ok() or error code
  */
