@@ -8,6 +8,7 @@
 #include <libknot/dname.h>
 #include <libknot/codes.h>
 
+#include "lib/rules/api.h"
 #include "lib/selection.h"
 #include "lib/zonecut.h"
 
@@ -26,7 +27,7 @@ struct kr_qflags {
 	bool AWAIT_IPV6 : 1;     /**< Query is waiting for AAAA address. */
 	bool AWAIT_CUT : 1;      /**< Query is waiting for zone cut lookup */
 	bool NO_EDNS : 1;        /**< Don't use EDNS. */
-	bool CACHED : 1;         /**< Query response is cached. */
+	bool CACHED : 1;         /**< Query response is cached (or generated locally). */
 	bool NO_CACHE : 1;       /**< No cache for lookup; exception: finding NSs and subqueries. */
 	bool EXPIRING : 1;       /**< Query response is cached but expiring.  See is_expiring(). */
 	bool ALLOW_LOCAL : 1;    /**< Allow queries to local or private address ranges. */
@@ -36,7 +37,8 @@ struct kr_qflags {
 	bool DNSSEC_INSECURE : 1;/**< Query response is DNSSEC insecure. */
 	bool DNSSEC_CD : 1;      /**< Instruction to set CD bit in request. */
 	bool STUB : 1;           /**< Stub resolution, accept received answer as solved. */
-	bool ALWAYS_CUT : 1;     /**< Always recover zone cut (even if cached). */
+	bool ALWAYS_CUT : 1;     /**< Always recover zone cut (even if cached).
+				  *   This flag might be broken and/or not useful anymore. */
 	bool DNSSEC_WEXPAND : 1; /**< Query response has wildcard expansion. */
 	bool PERMISSIVE : 1;     /**< Permissive resolver mode. */
 	bool STRICT : 1;         /**< Strict resolver mode. */
@@ -56,6 +58,7 @@ struct kr_qflags {
 	bool PKT_IS_SANE : 1;    /**< Set by iterator in consume phase to indicate whether
 				  * some basic aspects of the packet are OK, e.g. QNAME. */
 	bool DNS64_DISABLE : 1;  /**< Don't do any DNS64 stuff (meant for view:addr). */
+	bool PASSTHRU_LEGACY : 1;/**< Ignore local-data overrides/blocks for this kr_request. */
 };
 
 /** Combine flags together.  This means set union for simple flags. */
@@ -73,8 +76,12 @@ void kr_qflags_clear(struct kr_qflags *fl1, struct kr_qflags fl2);
 typedef int32_t (*kr_stale_cb)(int32_t ttl, const knot_dname_t *owner, uint16_t type,
 				const struct kr_query *qry);
 
-/**
- * Single query representation.
+/** A part of kr_request's resolution when sname and stype don't change.
+ *
+ * A kr_request can contain multiple kr_query parts.  A new one is needed when:
+ *   - CNAME jump occurs (or DNAME and similar)
+ *   - some other records are needed to proceed, e.g. DS/DNSKEY for validation or NS addresses
+ *   - generally, see kr_rplan_push() calls
  */
 struct kr_query {
 	struct kr_query *parent;
@@ -87,6 +94,12 @@ struct kr_query {
 	struct kr_qflags forward_flags;
 	uint32_t secret;
 	uint32_t uid; /**< Query iteration number, unique within the kr_rplan. */
+
+	/** Remaining limit; see kr_query::vld_limit_crypto docs */
+	int32_t  vld_limit_crypto_remains;
+	/** ::uid value to which this remaining limit applies. */
+	uint32_t vld_limit_uid;
+
 	uint64_t creation_time_mono; /* The time of query's creation (milliseconds).
 				 * Or time of creation of an oldest
 				 * ancestor if it is a subquery. */
@@ -95,6 +108,13 @@ struct kr_query {
 	struct timeval timestamp; /**< Real time for TTL+DNSSEC checks (.tv_sec only). */
 	struct kr_zonecut zone_cut;
 	struct kr_layer_pickle *deferred;
+	struct kr_query_data_src { // named struct to work around a bug in doc generator
+		bool initialized; /// !initialized -> all meaningless and zeroed
+		bool all_set;
+		uint8_t rule_depth; /// the number of labels for the apex
+		kr_rule_fwd_flags_t flags;
+		knot_db_val_t targets_ptr; /// pointer to targets inside rule DB
+	} data_src; /// information about "data source" for this sname+stype (+tags?)
 
 	/** Current xNAME depth, set by iterator.  0 = uninitialized, 1 = no CNAME, ...
 	 * See also KR_CNAME_CHAIN_LIMIT. */
