@@ -1,5 +1,12 @@
 #!/bin/bash
 # SPDX-License-Identifier: GPL-3.0-or-later
+
+# avoid confusing changes in ordering
+if ! locale | grep -q '^LC_COLLATE=.*\.UTF-8'; then
+	echo 'Error: you need to run this script with an .UTF-8 locale.'
+	exit 2
+fi
+
 set -o nounset -o xtrace
 
 function spdx_originator_to_authors {
@@ -15,7 +22,7 @@ TEMP_FILE="$(mktemp AUTHORS.XXXXXXXXXX)"
 # drop all names from the current file
 sed '/^People who contributed commits to our Git repo are/q' "${AUTHORS_FILE}" > "${TEMP_FILE}"
 # append to the new file
-git log --no-show-signature --format="%aN <%aE>" | sort -u | git check-mailmap --stdin | sort -u >> "${TEMP_FILE}"
+git log --format="%aN <%aE>" | sort -u | git check-mailmap --stdin | sort -u >> "${TEMP_FILE}"
 
 echo '' >> "${TEMP_FILE}"
 echo 'Knot Resolver source tree also bundles code and content published by:' >> "${TEMP_FILE}"
